/********************************************************
 * GAMEMENU						*
 *							*
 * cache.c						*
 * Version 1.0						*
 *							*
 * Changelog						*
 *							*
 * 1.0	-	2018/05/28	S.R.Usher		*
 *		Initial version.			*
 *							*
 ********************************************************/

#include <stdio.h>
#ifdef atari
#include <osbind.h>
#endif

#include "gamemenu.h"

unsigned long cacr_backup;

unsigned long cacr_value;

#ifdef atari
void cache_030_off();
void cache_030_restore();
#endif

void cache_off()
{
#ifdef atari
/*
 * Only do this if the machine have a 68030 CPU.
 *
 * If so do the real job in supervisor mode.
 */
	if (has_a_68030)
		Supexec(cache_030_off);
#endif
	return;
}

void cache_on()
{
#ifdef atari
/*
 * Only do this if the machine have a 68030 CPU.
 *
 * If so do the real job in supervisor mode.
 */
	if (has_a_68030)
		Supexec(cache_030_restore);
#endif
	return;
}

#ifdef atari
void cache_030_off()
{
/*
 * Save the current value of the cache control register and then set it to zero
 * to turn off all CPU caching.
 */
	cacr_backup = save_cacr();
	set_cacr(0L);
}

void cache_030_restore()
{
/*
 * Restore the saved value of the cache control register.
 */
	set_cacr(cacr_backup);
}

static __inline__ unsigned long save_cacr()
{
	register long retval __asm__ ("%d0");
	
	__asm__ __volatile__
	(
		"movec	cacr,d0\n\t"
		: "=r"(retval)
		:
		: "d0"
	);
}
		

static __inline__ void set_cacr(unsigned long val)
{
	__asm__ __volatile__
	(
		"movl	%0,d0\n\t"
		"movec	d0,cacr\n\t"
		: 
		: "g"(val)
		: "d0"
	);
}
#endif
