/********************************************************
 * GAMEMENU						*
 *							*
 * inifile.c						*
 * Version 1.0						*
 *							*
 * Changelog						*
 *							*
 * 1.0	-	2018/05/28	S.R.Usher		*
 *		Initial version.			*
 *							*
 ********************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

#include "gamemenu.h"

char *read_ini_file(struct ini_options *options)
{
	FILE *fp;
	char buffer[1024];
	char *equals, *value;
	int lineno = 0;
	int havegamedir = 0;
	int i;

/*
 * Make sure the scratch buffer always has a NULL at the end of any strings
 * written to it.
 */

	buffer[1023] = '\0';

/*
 * Set the defaults for the volume and speaker to negative (non-sensical) values
 * so the main routine knows if they've changed.
 */

	options->master_volume = -1;
	options->internal_speaker = -1;

/*
 * Open the file for reading. Note: we are looking in the current working directory
 * which is C:\ for AUTO programs.
 */

	if ((fp = fopen(INI_FILE, "r")) == NULL)
	{
		perror("Can't open init file 'INI_FILE'");
		return NULL;
	}

/*
 * Read in each line (with terminating characters removed) into the scratch buffer.
 */

	while(fgets_chomp(buffer, 1023, fp) != NULL)
	{
		lineno++;
/*
 * If the line begins with a '#' then it's a comment, skip the rest of the processing.
 */
 
		if (buffer[0] == '#') goto end;

/*
 * If the line doesn't contain an equals character it's invalid.
 * Give an error, abort and return.
 */

		if ((equals = strchr(buffer, '=')) == NULL)
		{
			fprintf(stderr,"Syntax error in INI file at line %d\n%s\n", lineno, buffer);
			fclose(fp);
			return NULL;
		}

/*
 * Split the line in two at the equals character and assign the second half to the 'value'
 */

		*equals = '\0';
		value = equals + 1;

/*
 * Note that the parsing is extremely crude. Any spaces will break things.
 *
 * We merely test to see if the first token is identical to what we are looking for
 * and semi-blindly assign the value.
 */

		if (!strcmp(buffer, "GAMEDIR"))
		{
			havegamedir = 1;
			strcpy(options->basedir, value);
		}

		if (!strcmp(buffer, "SPEAKER"))
		{
#ifdef DEBUG
			printf("Got SPEAKER option with value: '%s'\n", value);
#endif
			if (!strcmp(value, "ON"))
			{
#ifdef DEBUG
				printf("Turning it on.\n");
#endif
				options->internal_speaker = 1;
			}
			if (!strcmp(value, "OFF"))
			{
#ifdef DEBUG
				printf("Turning it off.\n");
#endif
				options->internal_speaker = 0;
			}
		}

/*
 * At least we do test to see if the voloume value is in range...
 */

		if (!strcmp(buffer, "VOLUME"))
		{
#ifdef DEBUG
			printf("Got VOLUME option with value: '%s'\n", value);
#endif
			if ((value[0] >= 48) && (value[0] <= 57))
				options->master_volume = atoi(value);
		}

end:		i = 0;
	}

	fclose(fp);

#ifdef DEBUG
	sleep(5);
#endif

/*
 * If the GAMEDIR value hasn't been set we have to tell the main program that
 * we've failed.
 */

	if (havegamedir == 1)
		return options->basedir;
	else
		return NULL;
}
