/********************************************************
 * GAMEMENU						*
 *							*
 * parse_csv.c						*
 * Version 1.0						*
 *							*
 * Changelog						*
 *							*
 * 1.0	-	2018/05/28	S.R.Usher		*
 *		Initial version.			*
 *							*
 ********************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

#include "gamemenu.h"

#define MAXLINE 255

#ifdef TESTING
int main()
{
	char filename[] = "GAMES.CSV";

	char *game_desc[MAXGAMES], *game_dir[MAXGAMES], *game_prog[MAXGAMES];
	int i, numgames;
	FILE *datafile;

	if ((datafile = fopen(filename, "ro")) == NULL)
	{
		perror("Opening datafile\n");
		exit(-1);
	}

	numgames = parse_csv_file(datafile, MAXGAMES, game_desc, game_dir, game_prog);

	if (numgames < 1)
	{
		printf("parse_csv_file returned error or no data. %d\n", numgames);
		exit(-1);
	}

	printf("Number of games found: %d\n", numgames);

	for (i = 0; i < numgames; i++)
	{
		printf("'%s'\t'%s'\t'%s'\n", game_desc[i], game_dir[i], game_prog[i]);
	}

	exit(0);
}
#endif TESTING

int parse_csv_file(fp, maxgames, game_desc, game_dir, game_prog)
FILE *fp;
int maxgames;
char *game_desc[];
char *game_dir[];
char *game_prog[];
{
	char line[MAXLINE], *newline;
	int i = 0, error;
	size_t linelen;
	char *firstcomma, *lastcomma;

/*
 * Read each line in the file until EOF or we've maxed out the entries, the end-of-line
 * characters are removed by fgetc_chomp() for us.
 */

	while((i < maxgames) && (fgets_chomp(line, MAXLINE - 1, fp) != NULL))
	{
/*
 * Assume there's an error unless otherwise notified.
 */
 
		error = 1;
/*
 * Duplicate the data so that we can just store the pointers to the data in the
 * arrays. It's quicker this way rather than having to copy each bit in turn.
 */

		linelen = strlen(line);

		if ((newline = (char *) malloc(linelen + 1)) == NULL)
		{
			perror("Error mallocing memory when reading CSV file. Aborting.");
			exit(-1);
		}

		memcpy(newline, line, linelen+1);

		linelen = strlen(newline);

/*
 * If we have a non-empty line then check that it has enough commas in it.
 *
 * Then replace the commas with NULLs to break up the string into three and
 * store the pointers in the arrays.
 */

		if (linelen > 0)
		{
			firstcomma = strchr(newline, ',');
			lastcomma = strrchr(newline, ',');

			if ((firstcomma != NULL) && (lastcomma != NULL))
			{
				if (firstcomma != lastcomma)
				{
					error = 0;

					*firstcomma = '\0';
					*lastcomma = '\0';

					game_desc[i] = newline;
					game_dir[i] = (char *)(firstcomma + 1);
					game_prog[i] = (char *)(lastcomma + 1);

					i++;
				}
			}
		}

/*
 * If there's an error print a report, including the line number and abort,
 * telling the main program there's been an error.
 */

		if (error == 1)
		{
			fprintf(stderr, "Error in CSV file at line %d\n", i + 1);
			return -1;
		}
	}
/*
 * Return the number of entries in the arrays.
 */
	return i;
}
