/********************************************************
 * GAMEMENU						*
 *							*
 * run_game.c						*
 * Version 1.0						*
 *							*
 * Changelog						*
 *							*
 * 1.0	-	2018/05/28	S.R.Usher		*
 *		Initial version.			*
 *							*
 ********************************************************/

#if !defined(atari) || defined(DEBUG)
#include <stdio.h>

void run_game(char *dir, char *prog)
{
	printf("exec %s/%s\n", dir, prog);
	return;
}
#else
#include <stdio.h>
#include <string.h>
#include <osbind.h>

#include "gamemenu.h"

void set_palmode();

short *palmode = (short *) 0x00000448;
unsigned char *vdureg = (unsigned char *) 0x00ff820a;

void run_game(char *dir, char *prog, int argc, char **argv, char **envp)
{
	int error;

/*
 * Attempt to change our working directory to that of the game.
 * Abort if we can't.
 */

	if (chdir(dir) < 0)
	{
		perror("Changing to game directory");
		return;
	}

/*
 * Some games only work if they think the machine is running
 * on a European (50Hz) TV. So change the system variable to
 * fool them.
 */
 
#ifndef DEBUG
	Supexec(set_palmode);
#endif

/*
 * Set up the exec process and turn the cache off, if there is one.
 */
 
	argv[0] = prog;

#ifndef DEBUG
	cache_off();
	
	error = execve(prog, argv, envp);

/*
 * Oh. The attempt at running the game failed. Print the error,
 * reset the cache to how it was previously and return.
 */

	perror(prog);

	cache_on();
#else
	printf("execve(%s, argv, envp)\n", prog);
#endif
}

void set_palmode()
{
	unsigned char value;
	
	*palmode = 3;
	value = *vdureg;
	value = value | 0x02;
	*vdureg = value;
}
#endif /* atari */
