/********************************************************
 * GAMEMENU						*
 *							*
 * screen_mode.c					*
 * Version 1.0						*
 *							*
 * Changelog						*
 *							*
 * 1.0	-	2018/05/28	S.R.Usher		*
 *		Initial version.			*
 *							*
 ********************************************************/

#include <stdio.h>

#ifdef atari
#include <osbind.h>
#endif

#include "gamemenu.h"

char *physbase, *logbase;
int res;
int colours[] = { 0x007, 0x770, 0x770, 0x770,
			0x770, 0x770, 0x770, 0x770,
			0x770, 0x770, 0x770, 0x770,
			0x770, 0x770, 0x770, 0x770
		};
int orig_colours[16];

void set_screen_mode()
{
	int i;
#ifdef atari
/*
 * Get and store the current screen parameters.
 */

	physbase = (char *) Physbase();
	logbase = (char *) Logbase();
	res = Getrez();

/*
 * If we're not a TT then change to ST-Medium resolution.
 * Otherwise switch to TT-Medium as it looks nicer.
 */

	if (!is_a_tt)
		Setscreen(physbase, logbase, 1);
	else
		Setscreen(physbase, logbase, 4);

/*
 * Save the original palette colours and change them to ours.
 */

	for (i = 0; i < 16; i++)
	{
		orig_colours[i] = Setcolor(i, -1);
		Setcolor(i, colours[i]);
	}
#endif
	return;
}

void unset_screen_mode()
{
	int i;
#ifdef atari
/*
 * Restore the screen settings and palette back to the values we
 * stored above. Leave no traces.
 */

	for (i = 0; i < 16; i++)
		Setcolor(i, orig_colours[i]);

	Setscreen(physbase, logbase, res);
#endif
	return;
}
