/********************************************************
 * GAMEMENU						*
 *							*
 * sound.c						*
 * Version 1.0						*
 *							*
 * Changelog						*
 *							*
 * 1.0	-	2018/05/28	S.R.Usher		*
 *		Initial version.			*
 *							*
 ********************************************************/

#include <stdio.h>

#if !defined(atari) || defined(DEBUG)
void set_volume(int volume)
{
	printf("Setting volume to %d\n", volume);
	return;
}

void set_speaker(int onoff)
{
	printf("Turning speaker %s\n", onoff == 0 ? "on" : "off");
	return;
}
#else
#include <osbind.h>
#include <string.h>

/*
 * Define the MicroWire interface addresses.
 */

#define MICROWIRE_DATA_ADDR 0xff8922
#define MICROWIRE_MASK_ADDR 0xff8924

unsigned short *mw_mask = MICROWIRE_MASK_ADDR;
unsigned short *mw_data = MICROWIRE_DATA_ADDR;

/*
 * Define the bit mask values for the PSG Port A.
 */

#define PSG_PA_FLPS0SEL	0x01
#define PSG_PA_FLP0SEL	0x02
#define PSG_PA_FLP1SEL	0x04
#define PSG_PA_MFPRTS	0x08
#define PSG_PA_MFPDTR	0x10
#define PSG_PA_PPSTROBE	0x20
#define PSG_PA_TTSPKDIS	0x40
#define PSG_PA_TTSERSEL 0x80

unsigned short value;

void do_microwire();

void set_volume(int volume)
{
/*
 * Add the volume (value between 0 and 40) to the
 * MicroWire command template.
 */
	value = 0x04c0 + volume;

/*
 * Send it on its way.
 */

	Supexec(do_microwire);

	return;
}

void do_microwire()
{
	int i;
/*
 * Set the MicroWire command mask to the chip
 */
	unsigned short mask = 0x07ff;

	*mw_mask = mask;

/*
 * Wait until it's been shifted down the line and the value returns.
 */

	while(*mw_mask != mask) i++;

/*
 * Now we can post the data down the line.
 */

	*mw_data = value;
}

/*
 * The TT Speaker control is the opposite of what you would think.
 * You have to SET bit 6 of the PSG Port A to turn it off.
 *
 * Note: I hit a "feature" of the way the mintlibs' inline assembler
 * versions of the system calls in osbind.h work.
 *
 * You need to send an unsigned short as the GCC seems not to be able 
 * to do type conversion for you.
 */

void set_speaker(int onoff)
{
	unsigned short mask;
	
#ifdef DEBUG2
	printf("\n\nset_speaker: onoff = %d\n", onoff);
#endif

	if (onoff == 1)
	{
		mask = 0xffff - PSG_PA_TTSPKDIS;
		Offgibit(mask);
	}
	else
	{
		mask = PSG_PA_TTSPKDIS;
		Ongibit(mask);
	}

#ifdef DEBUG2
	printf("\n\n");
	sleep(5);
#endif
	return;
}

#endif
